#include "core.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <decoders/decoder.h>
#include <core/x_core.h>
#include <core/macros/mouse.h>

static inline void timeout(void)
{
	x_keyboard_data.n_keys = 0;
	x_keyboard_data.mod_byte = 0;
	X_MOUSE_RESET();
	x_ir_reader.log = false;
}

x_ir_reader_t x_ir_reader =
{
	.signal = false,
	.log = false,
	.data = {0},
	.data_pos = 0,
	.keyboard_changed = false,
	.key_ignore_time = 0
};

#if (ARCH == ARCH_XMEGA)

#include <usb/xmega/Q/osc/osc.h>
#include <usb/LUFA/Drivers/USB/USB.h>
#include <usb/LUFA/Platform/Platform.h>
#include <usb/LUFA/Drivers/USB/Class/Device/HIDClassDevice.h>
#include <usb/xmega/Descriptors.h>
#include <usb/xmega/usb.h>

static inline void usb_init(void)
{
	int2x_ppl_USB48(((F_CPU/100000) + 10)/20); //Takt automatisch nach F_CPU setzen
	USB_Init();
	
	CDC_Device_CreateStream(&VirtualSerial_CDC_Interface, stdout);
}

static inline void usb_loop(void)
{
	CDC_Device_ReceiveByte(&VirtualSerial_CDC_Interface);
	
	CDC_Device_USBTask(&VirtualSerial_CDC_Interface);
	HID_Device_USBTask(&Mouse_HID_Interface);
	HID_Device_USBTask(&Keyboard_HID_Interface);
	USB_USBTask();
}


#elif (ARCH == ARCH_AVR8)

#include <avr/wdt.h>
#include <usb/v-usb/usbdrv.h>
#include <usb/mega/usb.h>
#include <avr/io.h>
#include <util/delay.h>

#include <avr/iomx8.h>
#include <core/scancodes.h>

static inline void usb_loop(void)
{
	uint8_t i;
	static uint8_t mouse_buttons = 0xFF;
	static uint8_t n_keys = 0x00;
	static uint8_t modifiers = 0x00;
	wdt_reset();
	usbPoll();
	if(usbInterruptIsReady())
	{
		if(x_mouse_info.mouse_move_x || x_mouse_info.mouse_move_y || x_mouse_info.mouse_wheel || x_mouse_info.buttons_pressed != mouse_buttons)
		{
			mouse_report.report_id = 0x4d;
			mouse_report.axis_x = x_mouse_info.mouse_move_x;
			mouse_report.axis_y = x_mouse_info.mouse_move_y;
			mouse_report.wheel = x_mouse_info.mouse_wheel;
			mouse_report.buttons = x_mouse_info.buttons_pressed;
			x_mouse_info.mouse_move_y = 0;
			x_mouse_info.mouse_move_x = 0;
			x_mouse_info.mouse_wheel = 0;
			usbSetInterrupt((void *)&mouse_report, sizeof(mouse_report));
			mouse_buttons = x_mouse_info.buttons_pressed;
			return;
		}
		
		if(x_keyboard_data.n_keys > 0 || n_keys != x_keyboard_data.n_keys || modifiers != x_keyboard_data.mod_byte)
		{
			n_keys = x_keyboard_data.n_keys;
			modifiers = x_keyboard_data.mod_byte;
			
			if(!x_keyboard_data.n_keys)
			{
				MERLIN_LED2_TGL();
			}
			keyboard_report.report_id = 0x02;
			for(i = 0; i < x_keyboard_data.n_keys; i++)
			{
				//Alle gerade gedrueckten Keys
				keyboard_report.keycode[i] = x_keyboard_data.keys[i];
			}

			for( ; i < X_MAX_KEYS; i++)
			{
				keyboard_report.keycode[i] = 0;
			}

			//sowie aktive Modifiers
			keyboard_report.modifier = x_keyboard_data.mod_byte;
			
			
			usbSetInterrupt((void *)&keyboard_report, sizeof(keyboard_report));
			return;
		}
		
// 		usbSetInterrupt((void *)&keyboard_report, 0);
	}
}

static inline void usb_init(void)
{
	wdt_enable(WDTO_1S);
	/* Even if you don't use the watchdog, turn it off here. On newer devices,
	 * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
	 */
	/* RESET status: all port bits are inputs without pull-up.
	 * That's the way we need D+ and D-. Therefore we don't need any
	 * additional hardware initialization.
	 */
	uint8_t i;
	usbInit();
	usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
	
	i = 0;
	while(--i)
	{             /* fake USB disconnect for > 250 ms */
		wdt_reset();
		_delay_ms(1);
	}
	usbDeviceConnect();
}

#else
#error "ERROR: Arch not supported"
#endif

void merlin_global_init(void)
{
	
	MERLIN_LEDS_SETUP();
	
	

	usb_init();
	
	_delay_ms(1);
	
	MERLIN_TIMER_SETUP();
	// 	MERLIN_ENABLE_TIMER_IRQ();
	
	MERLIN_EXT_IRQ_SETUP();
	MERLIN_EXT_IRQ_ENABLE();
	
	MERLIN_ENABLE_IRQ_GLOBAL();
	
	x_core_init();
}

#define TIMER_CLKS_MS (F_CPU/( 1000 * X_TIMER_DIV))

void merlin_main_loop(void)
{
	usb_loop();
	
	static uint32_t time_gone = 0;
	static uint32_t last_time_out = 0;
	static uint16_t last_timer = 0;
	uint16_t temp;
	MERLIN_TIMER_GETRES(temp);
	
	time_gone += (uint16_t) temp - last_timer;
	
	last_timer = temp;
	
	static uint8_t x_codes[X_MAX_KEYS];
	
	int8_t tmp;
	
	
	//if we have no timeout we do not need to do nore stuff here
	if(time_gone - last_time_out < TIMER_CLKS_MS * X_TIMEOUT_MS)
	{
		return;
	}
	
	if(x_ir_reader.key_ignore_time > 0)
	{
		x_ir_reader.key_ignore_time--;
	}

// 	MERLIN_LED1_TGL();
	
	//Set the new last timeout time
	last_time_out += TIMER_CLKS_MS * X_TIMEOUT_MS;
	
	//We are always calling users routine
	tmp = signal_timeout(x_codes);
	
	//And if he think so we need to translate the new tables
	if(tmp != -1)
	if(tmp >= 0)
	{
		translate(x_codes, tmp);
	}

	//If we are not logging or not finish with logging we can return here
	if(!x_ir_reader.log || x_ir_reader.signal)
	{
		x_ir_reader.signal = false;
		return;
	}



	if(x_ir_reader.data_pos > 5) //Ignore incomplete codes
	{
		//We need to make this decode also if we are ignoring signals for now
		//Let the user refresh his tables
		MERLIN_EXT_IRQ_DISABLE();
		tmp = decode(x_codes);
		x_ir_reader.log = false;
		x_ir_reader.data_pos = 0;
		MERLIN_EXT_IRQ_ENABLE();
		
		x_ir_reader.signal = false;
		if(tmp >= 0 && !x_ir_reader.key_ignore_time)
		{
			translate(x_codes, tmp);
		}
	}
	else
	{
		x_ir_reader.signal = false;
		x_ir_reader.log = false;
		x_ir_reader.data_pos = 0;
	}

}

//TODO: ein timer wäre schöner aber v-usb mag das bisher leider nicht

ISR(MERLIN_EXT_IRQ_VECT)
{
	uint16_t res;
	MERLIN_TIMER_GETRES(res);
	if(x_ir_reader.data_pos < X_MAX_DATA_LENGTH)
	{
		x_ir_reader.data[x_ir_reader.data_pos] = res;
		x_ir_reader.data_pos++;
	}
	x_ir_reader.signal = true;  //Has clock this perriode
	x_ir_reader.log = true;     //Currently reading
}