#include "../decoder.h"
#include <core/core.h>
#include <core/x_helpers.h>
#include <core/macros/debug.h>
#include <core/x_scancodes.h>
#include <avr/pgmspace.h>

#include <core/macros/mouse.h>

#ifndef F_CPU
#define F_CPU 32000000L
#endif

//µS to clocks
#define US_TO_TIMER_CLKS(us)   (us * (F_CPU/1000000L) / X_TIMER_DIV)

#define HEADER_LOW_MIN         US_TO_TIMER_CLKS(1500L)
#define HEADER_LOW_MAX         US_TO_TIMER_CLKS(2400L)

#define HEADER_HIGH_MIN        US_TO_TIMER_CLKS(800L)
#define HEADER_HIGH_MAX        US_TO_TIMER_CLKS(1200L)

#define TRANSMISSION_LOW_MIN   US_TO_TIMER_CLKS(190L) //Norminal ~320
#define TRANSMISSION_LOW_MAX   US_TO_TIMER_CLKS(430L)

#define TRANSMISSION_HIGH_1    US_TO_TIMER_CLKS(470L)

extern const uint8_t merlin_keycodes[128] PROGMEM;

uint8_t ir_keys_active[6];
uint8_t ir_keys_active_count = 0;

uint16_t timeout_time = 0;

typedef struct
{
	unsigned mouse_left:1;
	unsigned mouse_right:1;
	unsigned reserved2:1;
	unsigned mouse_only_cmd:1;
	unsigned mouse_y_neg:1;
	unsigned mouse_x_neg:1;	
	unsigned reserved:2;
} mouse_byte_t;

uint8_t get_x_code(uint8_t ir_code)
{
	if(ir_code < sizeof(merlin_keycodes))
	{
		return pgm_read_byte(merlin_keycodes + ir_code);
	}
	else if(ir_code == 134)
	{
		return X_MACRO_SELECT;
	}
	else if(ir_code == 160)
	{
		return X_SC_LEFT_GUI;
	}
	else
	{
		printf_debug("key %d not mapped\n");
		return 0;
	}
}


int8_t signal_timeout(uint8_t *result)
{
	uint8_t i, tmp;
	//This special timeout control is needed because
	//the keyboard is sending the stop code only once.
	//If stop code will not be fetched by the receiver
	//the key would be pressed forever.
	//To avoid that there is no way than that dirty
	//workaround.
	
	//The slow timeout affect all keys
	if(timeout_time <= 1001)
	{
		timeout_time++;
		if(timeout_time == 1000)
		{
			printf_debug("big timeout\n");
			//Reset all keys
			
			ir_keys_active_count = 0;
			return 0;
		}
		
		//The fast timeout affect all keys but not the modifiers
		if(timeout_time == 160)
		{
			printf_debug("small timeout\n");
			for(i = 0; i < ir_keys_active_count; i++)
			{
				tmp = get_x_code(ir_keys_active[i]);
				//It is not a modifier
				if(tmp < X_SC_LEFT_CONTROL || tmp > X_SC_RIGHT_GUI)
				{
					//Replace code with the last and remove the last
					ir_keys_active[i] = ir_keys_active[ir_keys_active_count - 1];
					ir_keys_active_count--;
				}
			}
			return ir_keys_active_count;
		}
	}
	return -1;
}




int8_t decode(uint8_t *result)
{

	//Grep header, as long we have no error correction it's need to be 84 bytes long
	if(x_ir_reader.data_pos != 84)
	{
		printf_debug("rp = %d != 84\n", x_ir_reader.data_pos);
		return -1;
	}
	uint8_t i = 0, bit, byte;
	
	uint16_t t1, t2;
	//Header LOW is very short maybe we should ignore if we not get it
	t1 = X_GET_NEXT_PULSE(i);
	i++;
	t2 = X_GET_NEXT_PULSE(i);
	
	if( t1 < HEADER_LOW_MIN || t2 < HEADER_HIGH_MIN || t1 > HEADER_LOW_MAX || t2 > HEADER_HIGH_MAX)
	{
		printf_debug("header damaged %u %u, min: %u\n", t1, t2, HEADER_LOW_MIN);
		return -1;
	}
	
	timeout_time = 0;
	
	uint8_t data[5] = {0, 0, 0, 0, 0};
	bit = 0;
	byte = 0;
	bool found = false;
	for(i = 2; i < 82 - 1; i++)
	{
		t1 = X_GET_NEXT_PULSE(i);
		
		if(t1 < TRANSMISSION_LOW_MIN || t1 > TRANSMISSION_LOW_MAX)
		{
			printf_debug("transmission %d\n", t1 >> 5);
			return -1;
		}
		
		i++;
		
		//Shifting in from left because signal comes with least significant bit first
		data[byte] >>= 1;
		//if signal length higher than the threshold we can write an "one" to the bit position
		if(X_GET_NEXT_PULSE(i) > TRANSMISSION_HIGH_1)
		{
			data[byte] |= 0x80;
		}
		
		if(bit == 7)
		{
			bit = 0;
			byte++;
			continue;
		}
		bit++;
	}
	
	if(data[3] != (uint8_t) ~data[4])
	{
		printf_debug("checksum failed %x, %x\n", data[3], data[4]);
		return -1;
	}
	
	printf_debug("%x %x %x %d\n", data[0], data[1], data[2], data[3]);
	
	//We are ignoring the first nibble of the header
	if(data[0] == 0x0F)
	{
		printf_debug("header failed\n");
		return -1;
	}
	
	if(!data[1])
	{
		//data[2] holds mouse command and data[3] holds keycode not inverted
		if(data[2] == 0)
		{
			for(i = 0; i < ir_keys_active_count; i++)
			{
				if(ir_keys_active[i] == data[3])
				{
					found = true;
					break;
				}
			}
			
			//If not in list we add the new active keycode
			if(!found)
			{
				ir_keys_active[ir_keys_active_count] = data[3];
				ir_keys_active_count++;
			}
		}
		else if(data[2] == 0xF0) //Stop code so we need to remove a code from the list
		{
			for(i = 0; i < ir_keys_active_count; i++)
			{
				if(ir_keys_active[i] == data[3])
				{
					//Replace the key we want to delete with the last key in list
					//and delete the last.
					ir_keys_active[i] = ir_keys_active[ir_keys_active_count - 1];
					ir_keys_active_count--;
					break;
				}
			}
		}
	}


	for(i = 0; i < ir_keys_active_count; i++)
	{
		result[i] = get_x_code(ir_keys_active[i]);

		printf_debug("%d->%d ", ir_keys_active[i], result[i]);
	}
	
	if(data[1])
	{
		if(data[2] || data[3])
		{
			result[i] = MOUSE_LEFT;
			i++;
			x_mouse_info.mouse_move_x += (int8_t)data[2] << 1;
			x_mouse_info.mouse_move_y -= (int8_t)data[3] << 1;
		}
		if(((mouse_byte_t *) data + 1)->mouse_left)
		{
			printf_debug("b1\n");
			result[i] = B1;
			i++;
		}
		if(((mouse_byte_t *) data + 1)->mouse_right)
		{
			printf_debug("b2\n");
			result[i] = B2;
			i++;
		}
	}

	return i;
}


//Keycode Map
const uint8_t merlin_keycodes[] PROGMEM =
{
	X_NOT_MAPPED, /*0*/
	X_SC_GRAVE_ACCENT_AND_TILDE, /*1*/
	X_SC_1_AND_EXCLAMATION, /*2*/
	X_SC_2_AND_AT, /*3*/
	X_SC_3_AND_HASHMARK, /*4 a*/
	X_SC_4_AND_DOLLAR, /*5 b*/
	X_SC_5_AND_PERCENTAGE, /*6 c*/
	X_SC_6_AND_CARET, /*7 d*/
	X_SC_7_AND_AMPERSAND, /*8 e*/
	X_SC_8_AND_ASTERISK, /*9 f*/
	X_SC_9_AND_OPENING_PARENTHESIS, /*10 g*/
	X_SC_0_AND_CLOSING_PARENTHESIS, /*11 h*/
	X_SC_MINUS_AND_UNDERSCORE, /*12 i*/
	X_SC_EQUAL_AND_PLUS, /*13 j*/
	X_NOT_MAPPED, /*14 k*/
	X_SC_BACKSPACE, /*15 l*/
	X_SC_TAB, /*16 m*/
	X_SC_Q, /*17*/
	X_SC_W, /*18*/
	X_SC_E, /*19*/
	X_SC_R, /*20*/
	X_SC_T, /*21*/
	X_SC_Y, /*22*/
	X_SC_U, /*23*/
	X_SC_I, /*24*/
	X_SC_O, /*25*/
	X_SC_P, /*26*/
	X_SC_OPENING_BRACKET_AND_OPENING_BRACE, /*27*/
	X_SC_CLOSING_BRACKET_AND_CLOSING_BRACE, /*28*/
	X_NOT_MAPPED, /*29*/
	X_SC_CAPS_LOCK, /*30*/
	X_SC_A, /*31*/
	X_SC_S, /*32*/
	X_SC_D, /*33*/
	X_SC_F, /*34*/
	X_SC_G, /*35*/
	X_SC_H, /*36*/
	X_SC_J, /*37*/
	X_SC_K, /*38*/
	X_SC_L, /*39*/
	X_SC_SEMICOLON_AND_COLON, /*40*/
	X_SC_APOSTROPHE_AND_QUOTE, /*41*/
	X_SC_BACKSLASH_AND_PIPE, /*42*/
	X_SC_ENTER, /*43*/
	X_SC_LEFT_SHIFT, /*44*/
	X_SC_NON_US_BACKSLASH_AND_PIPE, /*45*/
	X_SC_Z, /*46*/
	X_SC_X, /*47*/
	X_SC_C, /*48*/
	X_SC_V, /*49*/
	X_SC_B, /*50*/
	X_SC_N, /*51*/
	X_SC_M, /*52*/
	X_SC_COMMA_AND_LESS_THAN_SIGN, /*53*/
	X_SC_DOT_AND_GREATER_THAN_SIGN, /*54*/
	X_SC_SLASH_AND_QUESTION_MARK, /*55*/
	X_NOT_MAPPED, /*56*/
	X_SC_RIGHT_SHIFT, /*57*/
	X_SC_LEFT_CONTROL, /*58*/
	X_NOT_MAPPED, /*59*/
	X_SC_LEFT_ALT, /*60*/
	X_SC_SPACE, /*61*/
	X_SC_RIGHT_ALT, /*62*/
	X_NOT_MAPPED, /*63*/
	X_NOT_MAPPED, /*64*/
	X_NOT_MAPPED, /*65*/
	X_NOT_MAPPED, /*66*/
	X_NOT_MAPPED, /*67*/
	X_NOT_MAPPED, /*68*/
	X_NOT_MAPPED, /*69*/
	X_NOT_MAPPED, /*70*/
	X_NOT_MAPPED, /*71*/
	X_NOT_MAPPED, /*72*/
	X_NOT_MAPPED, /*73*/
	X_NOT_MAPPED, /*74*/
	X_SC_INSERT, /*75*/
	X_SC_DELETE, /*76*/
	X_NOT_MAPPED, /*77*/
	X_NOT_MAPPED, /*78*/
	X_SC_LEFT_ARROW, /*79*/
	X_SC_HOME, /*80*/
	X_SC_END, /*81*/
	X_NOT_MAPPED, /*82*/
	X_SC_UP_ARROW, /*83*/
	X_SC_DOWN_ARROW, /*84*/
	X_SC_PAGE_UP, /*85*/
	X_SC_PAGE_DOWN, /*86*/
	X_NOT_MAPPED, /*87*/
	X_NOT_MAPPED, /*88*/
	X_SC_RIGHT_ARROW, /*89*/
	X_NOT_MAPPED, /*90*/
	X_NOT_MAPPED, /*91*/
	X_NOT_MAPPED, /*92*/
	X_NOT_MAPPED, /*93*/
	X_NOT_MAPPED, /*94*/
	X_NOT_MAPPED, /*95*/
	X_NOT_MAPPED, /*96*/
	X_NOT_MAPPED, /*97*/
	X_NOT_MAPPED, /*98*/
	X_NOT_MAPPED, /*99*/
	X_NOT_MAPPED, /*100*/
	X_NOT_MAPPED, /*101*/
	X_NOT_MAPPED, /*102*/
	X_NOT_MAPPED, /*103*/
	X_NOT_MAPPED, /*104*/
	X_NOT_MAPPED, /*105*/
	X_NOT_MAPPED, /*106*/
	X_NOT_MAPPED, /*107*/
	X_NOT_MAPPED, /*108*/
	X_NOT_MAPPED, /*109*/
	X_SC_ESCAPE, /*110*/
	X_NOT_MAPPED, /*111*/
	X_SC_F1, /*112*/
	X_SC_F2, /*113*/
	X_SC_F3, /*114*/
	X_SC_F4, /*115*/
	X_SC_F5, /*116*/
	X_SC_F6, /*117*/
	X_SC_F7, /*118*/
	X_SC_F8, /*119*/
	X_SC_F9, /*120*/
	X_SC_F10, /*121*/
	X_SC_F11, /*122*/
	X_SC_F12, /*123*/
	X_SC_PRINT_SCREEN, /*124*/
	X_NOT_MAPPED, /*125*/
	X_SC_POWER, /*126*/
};

