/*
             LUFA Library
     Copyright (C) Dean Camera, 2014.

  dean [at] fourwalledcubicle [dot] com
           www.lufa-lib.org
*/

/*
  Copyright 2014  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  The author disclaims all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

/** \file
 *
 *  Main source file for the Keyboard demo. This file contains the main tasks of
 *  the demo and is responsible for the initial application hardware configuration.
 */

#include "usb.h"
#include <decoders/decoder.h>
#include "Descriptors.h"
#include <core/x_core.h>
#include <core/core.h>
#include <core/macros/mouse.h>

/** LUFA CDC Class driver interface configuration and state information. This structure is
 *  passed to all CDC Class driver functions, so that multiple instances of the same class
 *  within a device can be differentiated from one another.
 */
USB_ClassInfo_CDC_Device_t VirtualSerial_CDC_Interface =
{
	.Config =
	{
		.ControlInterfaceNumber   = INTERFACE_ID_CDC_CCI,
		.DataINEndpoint           =
		{
			.Address                = CDC_TX_EPADDR,
			.Size                   = CDC_TXRX_EPSIZE,
			.Banks                  = 1,
		},
		.DataOUTEndpoint                =
		{
			.Address                = CDC_RX_EPADDR,
			.Size                   = CDC_TXRX_EPSIZE,
			.Banks                  = 1,
		},
		.NotificationEndpoint           =
		{
			.Address                = CDC_NOTIFICATION_EPADDR,
			.Size                   = CDC_NOTIFICATION_EPSIZE,
			.Banks                  = 1,
		},
	},
};

/** Buffer to hold the previously generated Keyboard HID report, for comparison purposes inside the HID class driver. */
static uint8_t PrevKeyboardHIDReportBuffer[sizeof(USB_KeyboardReport_Data_t)];

/** Buffer to hold the previously generated Mouse HID report, for comparison purposes inside the HID class driver. */
static uint8_t PrevMouseHIDReportBuffer[sizeof(USB_MouseReport_Data_t)];

/** LUFA HID Class driver interface configuration and state information. This structure is
 *  passed to all HID Class driver functions, so that multiple instances of the same class
 *  within a device can be differentiated from one another.
 */
USB_ClassInfo_HID_Device_t Mouse_HID_Interface =
{
	.Config =
	{
		.InterfaceNumber                = INTERFACE_ID_Mouse,
		.ReportINEndpoint               =
		{
			.Address                = MOUSE_EPADDR,
			.Size                   = HID_EPSIZE,
			.Banks                  = 1,
		},
		.PrevReportINBuffer             = PrevMouseHIDReportBuffer,
		.PrevReportINBufferSize         = sizeof(PrevMouseHIDReportBuffer),
	},
};


USB_ClassInfo_HID_Device_t Keyboard_HID_Interface =
{
	.Config =
	{
		.InterfaceNumber              = INTERFACE_ID_Keyboard,
		.ReportINEndpoint             =
		{
			.Address              = KEYBOARD_EPADDR,
			.Size                 = HID_EPSIZE,
			.Banks                = 1,
		},
		.PrevReportINBuffer           = PrevKeyboardHIDReportBuffer,
		.PrevReportINBufferSize       = sizeof(PrevKeyboardHIDReportBuffer),
	},
};



/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void)
{
	
}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void)
{
	
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
	bool ConfigSuccess = true;
	
	ConfigSuccess &= HID_Device_ConfigureEndpoints(&Mouse_HID_Interface);
	ConfigSuccess &= HID_Device_ConfigureEndpoints(&Keyboard_HID_Interface);
	ConfigSuccess &= CDC_Device_ConfigureEndpoints(&VirtualSerial_CDC_Interface);
	
	USB_Device_EnableSOFEvents();
	
	// 	LEDs_SetAllLEDs(ConfigSuccess ? LEDMASK_USB_READY : LEDMASK_USB_ERROR);
}

/** Event handler for the library USB Control Request reception event. */
void EVENT_USB_Device_ControlRequest(void)
{
	CDC_Device_ProcessControlRequest(&VirtualSerial_CDC_Interface);
	HID_Device_ProcessControlRequest(&Mouse_HID_Interface);
	HID_Device_ProcessControlRequest(&Keyboard_HID_Interface);
}

/** Event handler for the USB device Start Of Frame event. */
void EVENT_USB_Device_StartOfFrame(void)
{
	HID_Device_MillisecondElapsed(&Keyboard_HID_Interface);
	HID_Device_MillisecondElapsed(&Mouse_HID_Interface);
}

/** HID class driver callback function for the creation of HID reports to the host.
 *
 *  \param[in]     HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in,out] ReportID    Report ID requested by the host if non-zero, otherwise callback should set to the generated report ID
 *  \param[in]     ReportType  Type of the report to create, either HID_REPORT_ITEM_In or HID_REPORT_ITEM_Feature
 *  \param[out]    ReportData  Pointer to a buffer where the created report should be stored
 *  \param[out]    ReportSize  Number of bytes written in the report (or zero if no report is to be sent)
 *
 *  \return Boolean \c true to force the sending of the report, \c false to let the library determine if it needs to be sent
 */
bool CALLBACK_HID_Device_CreateHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo,
                                         uint8_t* const ReportID,
                                         const uint8_t ReportType,
                                         void* ReportData,
                                         uint16_t* const ReportSize)
{
	//Wollen keinen Report senden, wenn die andere Routine gerade Keycodes schreibt.

	static uint8_t n_keys = 0x00;
	static uint8_t modifiers = 0x00;
	
	if (HIDInterfaceInfo == &Keyboard_HID_Interface)
	{
		if(x_keyboard_data.n_keys > 0 || n_keys != x_keyboard_data.n_keys || modifiers != x_keyboard_data.mod_byte)
		{
			n_keys = x_keyboard_data.n_keys;
			modifiers = x_keyboard_data.mod_byte;
			
			USB_KeyboardReport_Data_t* KeyboardReport = (USB_KeyboardReport_Data_t*)ReportData;

			
			uint8_t i = 0;
			//Ansonsten holen wir die Daten:
			for(i = 0; i < x_keyboard_data.n_keys; i++)
			{
				//Alle gerade gedrueckten Keys
				KeyboardReport->KeyCode[i] = x_keyboard_data.keys[i];
			}
			//sowie aktive Modifiers
			KeyboardReport->Modifier = x_keyboard_data.mod_byte;


			*ReportSize = sizeof(USB_KeyboardReport_Data_t);
			return true;
		}
		else
		{
			*ReportSize = 0;
			return false;
		}
	}
#if X_MOUSE > 0

	USB_MouseReport_Data_t* MouseReport = (USB_MouseReport_Data_t*)ReportData;

	
	static uint8_t buttons = 0;
	
	if((x_mouse_info.buttons_pressed != buttons) || x_mouse_info.mouse_move_x || x_mouse_info.mouse_move_y || x_mouse_info.mouse_wheel)
	{
		buttons = x_mouse_info.buttons_pressed;
		
		MouseReport->Y = MAX(-X_MAX_MOUSE_MOVEMENT, MIN(X_MAX_MOUSE_MOVEMENT, x_mouse_info.mouse_move_y));
		x_mouse_info.mouse_move_y -= MouseReport->Y;
		
		
		MouseReport->X = MAX(-X_MAX_MOUSE_MOVEMENT, MIN(X_MAX_MOUSE_MOVEMENT, x_mouse_info.mouse_move_x));
		x_mouse_info.mouse_move_x -= MouseReport->X;
		
		MouseReport->Wheel = MAX(-X_MAX_MOUSE_MOVEMENT, MIN(X_MAX_MOUSE_MOVEMENT, x_mouse_info.mouse_wheel));
		x_mouse_info.mouse_wheel -= MouseReport->Wheel;
		
		MouseReport->Button = x_mouse_info.buttons_pressed;
		
		
		*ReportSize = sizeof(USB_MouseReport_Data_t);
		return true;
	}

#endif
	*ReportSize = 0;
	return false;

}

/** HID class driver callback function for the processing of HID reports from the host.
 *
 *  \param[in] HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in] ReportID    Report ID of the received report from the host
 *  \param[in] ReportType  The type of report that the host has sent, either HID_REPORT_ITEM_Out or HID_REPORT_ITEM_Feature
 *  \param[in] ReportData  Pointer to a buffer where the received report has been stored
 *  \param[in] ReportSize  Size in bytes of the received HID report
 */
void CALLBACK_HID_Device_ProcessHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo,
                                          const uint8_t ReportID,
                                          const uint8_t ReportType,
                                          const void* ReportData,
                                          const uint16_t ReportSize)
{
	
}

